@@ -13,37 +13,38 @@ class Conversion(Component):
     |br| @author: FINE Developer Team (FZJ IEK-3)
     """
 
-    def __init__(self, 
-                 esM, 
-                 name, 
-                 physicalUnit, 
-                 commodityConversionFactors, 
+    def __init__(self,
+                 esM,
+                 name,
+                 physicalUnit,
+                 commodityConversionFactors,
                  hasCapacityVariable=True,
-                 capacityVariableDomain='continuous', 
-                 capacityPerPlantUnit=1, 
+                 capacityVariableDomain='continuous',
+                 capacityPerPlantUnit=1,
                  linkedConversionCapacityID=None,
-                 hasIsBuiltBinaryVariable=False, 
+                 hasIsBuiltBinaryVariable=False,
                  bigM=None,
-                 operationRateMax=None, 
-                 operationRateFix=None, 
+                 operationRateMax=None,
+                 operationRateFix=None,
                  tsaWeight=1,
-                 locationalEligibility=None, 
-                 capacityMin=None, 
-                 capacityMax=None, 
-                 partLoadMin=None, 
+                 locationalEligibility=None,
+                 capacityMin=None,
+                 capacityMax=None,
+                 partLoadMin=None,
                  sharedPotentialID=None,
-                 capacityFix=None, 
+                 linkedQuantityID=None,
+                 capacityFix=None,
                  isBuiltFix=None,
-                 investPerCapacity=0, 
-                 investIfBuilt=0, 
-                 opexPerOperation=0, 
+                 investPerCapacity=0,
+                 investIfBuilt=0,
+                 opexPerOperation=0,
                  opexPerCapacity=0,
-                 opexIfBuilt=0, 
-                 QPcostScale=0, 
-                 interestRate=0.08, 
-                 economicLifetime=10, 
+                 opexIfBuilt=0,
+                 QPcostScale=0,
+                 interestRate=0.08,
+                 economicLifetime=10,
                  technicalLifetime=None,
-                 yearlyFullLoadHoursMin=None, 
+                 yearlyFullLoadHoursMin=None,
                  yearlyFullLoadHoursMax=None):
         # TODO: allow that the time series data or min/max/fixCapacity/eligibility is only specified for
         # TODO: eligible locations
@@ -62,8 +63,9 @@ class Conversion(Component):
         :param commodityConversionFactors: conversion factors with which commodities are converted into each
             other with one unit of operation (dictionary). Each commodity which is converted in this component
             is indicated by a string in this dictionary. The conversion factor related to this commodity is
-            given as a float. A negative value indicates that the commodity is consumed. A positive value
-            indicates that the commodity is produced. Check unit consistency when specifying this parameter!
+            given as a float (constant), pandas.Series or pandas.DataFrame (time-variable). A negative value 
+            indicates that the commodity is consumed. A positive value indicates that the commodity is produced.
+            Check unit consistency when specifying this parameter!
             Examples:\n
             * An electrolyzer converts, simply put, electricity into hydrogen with an electrical efficiency
               of 70%. The physicalUnit is given as GW_electric, the unit for the 'electricity' commodity is
@@ -74,7 +76,7 @@ class Conversion(Component):
             GW_electric and the 'hydrogen' commodity is given in GW_hydrogen_lowerHeatingValue
             -> the commodityConversionFactors are defined as {'electricity':1,'hydrogen':-1/0.6}.\n
         :type commodityConversionFactors: dictionary, assigns commodities (string) to a conversion factors
-            (float)
+            (float, pandas.Series or pandas.DataFrame)
 
         **Default arguments:**
 
@@ -118,30 +120,31 @@ class Conversion(Component):
         :type opexPerOperation: positive (>=0) float or Pandas Series with positive (>=0) values.
             The indices of the series have to equal the in the energy system model specified locations.
         """
-        Component. __init__(self, 
-                            esM, 
-                            name, 
-                            dimension='1dim', 
+        Component. __init__(self,
+                            esM,
+                            name,
+                            dimension='1dim',
                             hasCapacityVariable=hasCapacityVariable,
-                            capacityVariableDomain=capacityVariableDomain, 
+                            capacityVariableDomain=capacityVariableDomain,
                             capacityPerPlantUnit=capacityPerPlantUnit,
-                            hasIsBuiltBinaryVariable=hasIsBuiltBinaryVariable, 
+                            hasIsBuiltBinaryVariable=hasIsBuiltBinaryVariable,
                             bigM=bigM,
-                            locationalEligibility=locationalEligibility, 
+                            locationalEligibility=locationalEligibility,
                             capacityMin=capacityMin,
-                            capacityMax=capacityMax, 
-                            partLoadMin=partLoadMin, 
-                            sharedPotentialID=sharedPotentialID, 
+                            capacityMax=capacityMax,
+                            partLoadMin=partLoadMin,
+                            sharedPotentialID=sharedPotentialID,
+                            linkedQuantityID=linkedQuantityID,
                             capacityFix=capacityFix,
-                            isBuiltFix=isBuiltFix, 
-                            investPerCapacity=investPerCapacity, 
+                            isBuiltFix=isBuiltFix,
+                            investPerCapacity=investPerCapacity,
                             investIfBuilt=investIfBuilt,
-                            opexPerCapacity=opexPerCapacity, 
-                            opexIfBuilt=opexIfBuilt, 
-                            QPcostScale=QPcostScale, 
+                            opexPerCapacity=opexPerCapacity,
+                            opexIfBuilt=opexIfBuilt,
+                            QPcostScale=QPcostScale,
                             interestRate=interestRate,
-                            economicLifetime=economicLifetime, 
-                            technicalLifetime=technicalLifetime, 
+                            economicLifetime=economicLifetime,
+                            technicalLifetime=technicalLifetime,
                             yearlyFullLoadHoursMin=yearlyFullLoadHoursMin,
                             yearlyFullLoadHoursMax=yearlyFullLoadHoursMax)
 
@@ -150,7 +153,21 @@ class Conversion(Component):
         utils.checkCommodityUnits(esM, physicalUnit)
         if linkedConversionCapacityID is not None:
             utils.isString(linkedConversionCapacityID)
+
         self.commodityConversionFactors = commodityConversionFactors
+        self.fullCommodityConversionFactors = {}
+        self.aggregatedCommodityConversionFactors = {}
+        self.processedCommodityConversionFactors = {}
+        for commod in self.commodityConversionFactors.keys():
+            if not isinstance(self.commodityConversionFactors[commod], (int, float)):
+                self.fullCommodityConversionFactors[commod] = utils.checkAndSetTimeSeriesConversionFactors(esM, commodityConversionFactors[commod], self.locationalEligibility)
+                self.aggregatedCommodityConversionFactors[commod] = None
+            elif isinstance(self.commodityConversionFactors[commod], (int, float)):
+                # self.fullCommodityConversionFactors[commod] = None
+                # self.aggregatedCommodityConversionFactors[commod] = None
+                self.processedCommodityConversionFactors[commod] = self.commodityConversionFactors[commod]
+            
+
         self.physicalUnit = physicalUnit
         self.modelingClass = ConversionModel
         self.linkedConversionCapacityID = linkedConversionCapacityID
@@ -159,6 +176,9 @@ class Conversion(Component):
         self.opexPerOperation = utils.checkAndSetCostParameter(esM, name, opexPerOperation, '1dim',
                                                                 locationalEligibility)
 
+        self.operationRateMax = operationRateMax
+        self.operationRateFix = operationRateFix
+
         # Set location-specific operation parameters: operationRateMax or operationRateFix, tsaweight
         if operationRateMax is not None and operationRateFix is not None:
             operationRateMax = None
@@ -166,12 +186,12 @@ class Conversion(Component):
                 warnings.warn('If operationRateFix is specified, the operationRateMax parameter is not required.\n' +
                               'The operationRateMax time series was set to None.')
 
-        self.fullOperationRateMax = utils.checkAndSetTimeSeries(esM, operationRateMax, locationalEligibility)
-        self.aggregatedOperationRateMax, self.operationRateMax = None, None
-
-        self.fullOperationRateFix = utils.checkAndSetTimeSeries(esM, operationRateFix, locationalEligibility)
-        self.aggregatedOperationRateFix, self.operationRateFix = None, None
-
+        self.fullOperationRateMax = utils.checkAndSetTimeSeries(esM, name, operationRateMax, locationalEligibility)
+        self.aggregatedOperationRateMax, self.processedOperationRateMax = None, None
+        
+        self.fullOperationRateFix = utils.checkAndSetTimeSeries(esM, name, operationRateFix, locationalEligibility)
+        self.aggregatedOperationRateFix, self.processedOperationRateFix = None, None
+       
         if self.partLoadMin is not None:
             if self.fullOperationRateMax is not None:
                 if ((self.fullOperationRateMax > 0) & (self.fullOperationRateMax < self.partLoadMin)).any().any():
@@ -207,14 +227,21 @@ class Conversion(Component):
         :param hasTSA: states whether a time series aggregation is requested (True) or not (False).
         :type hasTSA: boolean
         """
-        self.operationRateMax = self.aggregatedOperationRateMax if hasTSA else self.fullOperationRateMax
-        self.operationRateFix = self.aggregatedOperationRateFix if hasTSA else self.fullOperationRateFix
+        self.processedOperationRateMax = self.aggregatedOperationRateMax if hasTSA else self.fullOperationRateMax
+        self.processedOperationRateFix = self.aggregatedOperationRateFix if hasTSA else self.fullOperationRateFix
+        if self.fullCommodityConversionFactors != {}:
+            for commod in self.fullCommodityConversionFactors:
+                self.processedCommodityConversionFactors[commod] = self.aggregatedCommodityConversionFactors[commod] if hasTSA else self.fullCommodityConversionFactors[commod]
 
     def getDataForTimeSeriesAggregation(self):
         """ Function for getting the required data if a time series aggregation is requested. """
         weightDict, data = {}, []
         weightDict, data = self.prepareTSAInput(self.fullOperationRateFix, self.fullOperationRateMax,
                                                 '_operationRate_', self.tsaWeight, weightDict, data)
+        for commod in self.fullCommodityConversionFactors:
+            weightDict, data = self.prepareTSAInput(self.fullCommodityConversionFactors[commod], None,
+                                                    '_commodityConversionFactorTimeSeries' + str(commod) + '_',
+                                                    self.tsaWeight, weightDict, data)
         return (pd.concat(data, axis=1), weightDict) if data else (None, {})
 
     def setAggregatedTimeSeriesData(self, data):
@@ -226,6 +253,9 @@ class Conversion(Component):
         """
         self.aggregatedOperationRateFix = self.getTSAOutput(self.fullOperationRateFix, '_operationRate_', data)
         self.aggregatedOperationRateMax = self.getTSAOutput(self.fullOperationRateMax, '_operationRate_', data)
+        if self.fullCommodityConversionFactors != {}:
+            for commod in self.fullCommodityConversionFactors:
+                self.aggregatedCommodityConversionFactors[commod] = self.getTSAOutput(self.fullCommodityConversionFactors[commod], '_commodityConversionFactorTimeSeries'+ str(commod) + '_', data)
 
 
 class ConversionModel(ComponentModel):
@@ -299,7 +329,7 @@ class ConversionModel(ComponentModel):
         self.declareOperationBinarySet(pyM)
 
         # Declare operation mode sets
-        self.declareOperationModeSets(pyM, 'opConstrSet', 'operationRateMax', 'operationRateFix')
+        self.declareOperationModeSets(pyM, 'opConstrSet', 'processedOperationRateMax', 'processedOperationRateFix')
 
         # Declare linked components dictionary
         self.declareLinkedCapacityDict(pyM)
@@ -431,15 +461,31 @@ class ConversionModel(ComponentModel):
         :param commod: Name of the regarded commodity (commodities are defined in the EnergySystemModel instance)
         :param commod: string
         """
-        return any([(commod in comp.commodityConversionFactors and comp.commodityConversionFactors[commod] != 0)
+        return any([(commod in comp.processedCommodityConversionFactors and (comp.processedCommodityConversionFactors[commod] is not
+                                                                    None))
                     and comp.locationalEligibility[loc] == 1 for comp in self.componentsDict.values()])
 
     def getCommodityBalanceContribution(self, pyM, commod, loc, p, t):
-        """ Get contribution to a commodity balance. """
+        """ 
+        Get contribution to a commodity balance. 
+        
+        .. math::
+            
+            \\text{C}^{comp,comm}_{loc,p,t} =  \\text{conversionFactor}^{comp}_{comm} \cdot op_{loc,p,t}^{comp,op} 
+        
+        """
         compDict, abbrvName = self.componentsDict, self.abbrvName
         opVar, opVarDict = getattr(pyM, 'op_' + abbrvName), getattr(pyM, 'operationVarDict_' + abbrvName)
-        return sum(opVar[loc, compName, p, t] * compDict[compName].commodityConversionFactors[commod]
-                   for compName in opVarDict[loc] if commod in compDict[compName].commodityConversionFactors)
+
+        def getFactor(commodCommodityConversionFactors, loc, p, t):
+            if isinstance(commodCommodityConversionFactors, (int, float)):
+                return commodCommodityConversionFactors
+            else:
+                return commodCommodityConversionFactors[loc][p,t]
+
+        return sum(opVar[loc, compName, p, t] * getFactor(compDict[compName].processedCommodityConversionFactors[commod], loc,
+                                                          p, t)
+                   for compName in opVarDict[loc] if commod in compDict[compName].processedCommodityConversionFactors)
 
     def getObjectiveFunctionContribution(self, esM, pyM):
         """
@@ -481,10 +527,14 @@ class ConversionModel(ComponentModel):
         self.operationVariablesOptimum = optVal
 
         props = ['operation', 'opexOp']
-        units = ['[-]', '[' + esM.costUnit + '/a]']
-        tuples = [(compName, prop, unit) for compName in compDict.keys() for prop, unit in zip(props, units)]
-        tuples = list(map(lambda x: (x[0], x[1], '[' + compDict[x[0]].physicalUnit + '*h/a]')
-                      if x[1] == 'operation' else x, tuples))
+        # Unit dict: Specify units for props
+        units = {props[0]: ['[-*h]', '[-*h/a]'],
+                 props[1]: ['[' + esM.costUnit + '/a]']}
+        # Create tuples for the optSummary's multiIndex. Combine component with the respective properties and units.
+        tuples = [(compName, prop, unit) for compName in compDict.keys() for prop in props for unit in units[prop]]
+        # Replace placeholder with correct unit of component
+        tuples = list(map(lambda x: (x[0], x[1], x[2].replace("-", compDict[x[0]].physicalUnit))
+            if x[1] == 'operation' else x, tuples))
         mIndex = pd.MultiIndex.from_tuples(tuples, names=['Component', 'Property', 'Unit'])
         optSummary = pd.DataFrame(index=mIndex, columns=sorted(esM.locations)).sort_index()
 
@@ -493,6 +543,8 @@ class ConversionModel(ComponentModel):
             ox = opSum.apply(lambda op: op * compDict[op.name].opexPerOperation[op.index], axis=1)
             optSummary.loc[[(ix, 'operation', '[' + compDict[ix].physicalUnit + '*h/a]') for ix in opSum.index],
                            opSum.columns] = opSum.values/esM.numberOfYears
+            optSummary.loc[[(ix, 'operation', '[' + compDict[ix].physicalUnit + '*h]') for ix in opSum.index],
+                           opSum.columns] = opSum.values
             optSummary.loc[[(ix, 'opexOp', '[' + esM.costUnit + '/a]') for ix in ox.index], ox.columns] = \
                 ox.values/esM.numberOfYears
 
